<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Block;

use Magento\Framework\View\Element\Template\Context;
use Magento\Framework\Encryption\EncryptorInterface;

class Meta extends \Magento\Framework\View\Element\Template
{
    /**
     * @var EncryptorInterface
     */
    protected $encryptor;

    /**
     * Constructor
     *
     * @param Context $context
     * @param EncryptorInterface $encryptor
     * @param array $data
     */
    public function __construct(
        Context $context,
        EncryptorInterface $encryptor,
        array $data = []
    ) {
        $this->encryptor = $encryptor;
        parent::__construct($context, $data);
    }

    /**
     * Get content
     *
     * @return string
     */
    public function getContent()
    {
        try {
            return sha1($this->getHost());
        } catch (\Exception $e) {
            return '';
        }
    }

    /**
     * Get current host
     *
     * @return string
     */
    public function getHost()
    {
        $url = str_replace(
            ['http://', 'https://'],
            '',
            $this->getBaseUrl()
        );

        return rtrim($url, '/');
    }
}
