<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\ApiManager;
use WriteTextAI\WriteTextAI\ViewModel\Premium;

class Add extends Action
{
    public const ADMIN_RESOURCE = 'WriteTextAI_WriteTextAI::add_keywords';

    /**
     * @var ApiManager
     */
    protected $apiManager;

    /**
     * @var Premium
     */
    protected $premium;

    /**
     * Constructor
     *
     * @param Context $context
     * @param ApiManager $apiManager
     * @param Premium $premium
     */
    public function __construct(
        Context $context,
        ApiManager $apiManager,
        Premium $premium
    ) {
        $this->apiManager = $apiManager;
        $this->premium = $premium;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }

        try {
            $keywords = $this->getRequest()->getParam('keywords');
            $keyword = $this->getRequest()->getParam('keyword');
            $productId = $this->getRequest()->getParam('product_id');
            $storeId = $this->getRequest()->getParam('store_id');

            $newKeywords = $keywords;
            array_push($newKeywords, $keyword);
            $semanticKeywords = $this->apiManager->getSemanticKeywords($storeId, $productId, $newKeywords);

            $texts = $semanticKeywords['texts'];
            $textIndex = array_search($keyword, array_column($texts, 'text'));
            $data = [
                'text' => $keyword,
                'values' => $texts[$textIndex]['values'] ?? []
            ];

            $keywordsTexts = array_column($texts, 'text');
            if (!empty($keywordsTexts)) {
                $audience = $this->apiManager->getSuggestedAudience($storeId, $productId, $keywordsTexts);
            }
            
            $result = [
                'success' => true,
                'data' => $data,
                'selected_audience' => $audience['selected'] ?? [],
                'suggested_audience' => $audience['values'] ?? []
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage(),
                'api_response' => $semanticKeywords
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
