<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\ViewModel\Premium;
use WriteTextAI\WriteTextAI\Model\Api\Session as ApiSession;
use WriteTextAI\WriteTextAI\Model\SettingManager;
use WriteTextAI\WriteTextAI\Model\Api\Keywords as ApiKeywords;
use WriteTextAI\WriteTextAI\Model\KeywordIdeaFiltersManager;
use WriteTextAI\WriteTextAI\Model\KeywordRankedFiltersManager;
use WriteTextAI\WriteTextAI\Model\KeywordCompetitorFiltersManager;
use WriteTextAI\WriteTextAI\Model\AiProductManager;

class AiPowered extends Action
{
    /**
     * @var Premium
     */
    protected $premium;

    /**
     * @var ApiSession
     */
    protected $apiSession;

    /**
     * @var SettingManager
     */
    protected $settingManager;

    /**
     * @var ApiKeywords
     */
    protected $apiKeywords;

    /**
     * @var KeywordIdeaFiltersManager
     */
    protected $keywordIdeaFiltersManager;

    /**
     * @var KeywordRankedFiltersManager
     */
    protected $keywordRankedFiltersManager;

    /**
     * @var KeywordCompetitorFiltersManager
     */
    protected $keywordCompetitorFiltersManager;

    /**
     * @var AiProductManager
     */
    protected $aiProductManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param Premium $premium
     * @param ApiSession $apiSession
     * @param SettingManager $settingManager
     * @param ApiKeywords $apiKeywords
     * @param KeywordIdeaFiltersManager $keywordIdeaFiltersManager
     * @param KeywordRankedFiltersManager $keywordRankedFiltersManager
     * @param KeywordCompetitorFiltersManager $keywordCompetitorFiltersManager
     * @param AiProductManager $aiProductManager
     */
    public function __construct(
        Context $context,
        Premium $premium,
        ApiSession $apiSession,
        SettingManager $settingManager,
        ApiKeywords $apiKeywords,
        KeywordIdeaFiltersManager $keywordIdeaFiltersManager,
        KeywordRankedFiltersManager $keywordRankedFiltersManager,
        KeywordCompetitorFiltersManager $keywordCompetitorFiltersManager,
        AiProductManager $aiProductManager
    ) {
        $this->premium = $premium;
        $this->apiSession = $apiSession;
        $this->settingManager = $settingManager;
        $this->apiKeywords = $apiKeywords;
        $this->keywordIdeaFiltersManager = $keywordIdeaFiltersManager;
        $this->keywordCompetitorFiltersManager = $keywordCompetitorFiltersManager;
        $this->keywordRankedFiltersManager = $keywordRankedFiltersManager;
        $this->aiProductManager = $aiProductManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);

        if (!$this->premium->getIsPremium()) {
            $response->setData([
                'success' => false,
                'is_premium' => false,
                'total_credits' => $this->premium->getTotalCredits(),
                'message' => __('Premium is required to do this action.')
            ]);

            return $response;
        }
        
        try {
            $params = $this->getRequest()->getParams();

            $rules = $this->settingManager->getSettings('rules');

            $currentUser = $this->apiSession->getCurrentUser();
            $regionIndependentLanguage = $this->apiSession->getRegionIndependentLanguage();

            $keywords = array_map(function ($text) use ($rules) {
                return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
            }, $params['keywords'] ?? []);

            $manualKeywords = array_map(function ($text) use ($rules) {
                return substr($text, 0, $rules['maxKeywordLength'] ?? 80);
            }, $params['manual_keywords'] ?? []);

            $apiResponse = $this->apiKeywords->aiPoweredAnalysis(
                $params['store_id'],
                $params['product_id'],
                $params['product_name'],
                $manualKeywords,
                $keywords,
                $params['location'],
                $regionIndependentLanguage,
                $params['product_url']
            );

            $this->keywordIdeaFiltersManager->deleteFilters($params['store_id'], $params['product_id']);
            $this->keywordRankedFiltersManager->deleteFilters($params['store_id'], $params['product_id']);
            $this->keywordCompetitorFiltersManager->deleteFilters($params['store_id'], $params['product_id']);
            $this->aiProductManager->setRefreshDate($params['product_id'], $params['store_id']);

            $result = [
                'success' => true,
                'api_response' => $apiResponse
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $result['is_premium'] = $this->premium->getIsPremium();
        $result['total_credits'] = $this->premium->getTotalCredits();

        $response->setData($result);

        return $response;
    }
}
