<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Edit\Keyword;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\KeywordRankedFiltersManager;
use WriteTextAI\WriteTextAI\Model\KeywordCompetitorFiltersManager;

class SaveFilter extends Action
{
    /**
     * @var KeywordRankedFiltersManager
     */
    protected $keywordRankedFiltersManager;

    /**
     * @var KeywordCompetitorFiltersManager
     */
    protected $keywordCompetitorFiltersManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param KeywordRankedFiltersManager $keywordRankedFiltersManager
     * @param KeywordCompetitorFiltersManager $keywordCompetitorFiltersManager
     */
    public function __construct(
        Context $context,
        KeywordRankedFiltersManager $keywordRankedFiltersManager,
        KeywordCompetitorFiltersManager $keywordCompetitorFiltersManager
    ) {
        $this->keywordRankedFiltersManager = $keywordRankedFiltersManager;
        $this->keywordCompetitorFiltersManager = $keywordCompetitorFiltersManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        
        try {
            $params = $this->getRequest()->getParams();

            switch ($params['type']) {
                case 'ranked':
                    $this->keywordRankedFiltersManager->saveKeywordRankedFilters(
                        $params['store_id'],
                        $params['product_id'],
                        $params['competition'] ?? [],
                        $params['search_volume_minimum'],
                        $params['search_volume_maximum'],
                        $params['competition_sort'],
                        $params['volume_sort'],
                        $params['rank_sort'],
                        $params['intent_sort']
                    );
                    break;
                case 'competitor':
                    $this->keywordCompetitorFiltersManager->saveKeywordCompetitorFilters(
                        $params['store_id'],
                        $params['product_id'],
                        $params['competition'] ?? [],
                        $params['search_volume_minimum'],
                        $params['search_volume_maximum'],
                        $params['competition_sort'],
                        $params['volume_sort'],
                        $params['rank_sort'],
                        $params['intent_sort']
                    );
                    break;
            }
                
            $result = [
                'success' => true
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }
}
