<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Controller\Adminhtml\Settings;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use WriteTextAI\WriteTextAI\Model\UserFieldsSelectedManager;

class Save extends Action
{
    /**
     * @var UserFieldsSelectedManager
     */
    protected $userFieldsSelectedManager;

    /**
     * Constructor
     *
     * @param Context $context
     * @param UserFieldsSelectedManager $userFieldsSelectedManager
     */
    public function __construct(
        Context $context,
        UserFieldsSelectedManager $userFieldsSelectedManager
    ) {
        $this->userFieldsSelectedManager = $userFieldsSelectedManager;
        parent::__construct($context);
    }

    /**
     * Execute
     *
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $response = $this->resultFactory->create(ResultFactory::TYPE_JSON);
        $mapping = [
            'highlighted' => UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHTED,
            'highlight_pronouns' => UserFieldsSelectedManager::ACTION_TYPE_HIGHLIGHT_PRONOUNS,
            'hide_bulk_settings' => UserFieldsSelectedManager::ACTION_TYPE_HIDE_BULK_SETTINGS,
            'enable_comparison_popup' => UserFieldsSelectedManager::ACTION_TYPE_ENABLE_COMPARISON_POPUP,
            'custom_tone' => UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_TONE,
            'custom_style' => UserFieldsSelectedManager::ACTION_TYPE_CUSTOM_STYLE,
            'style' => UserFieldsSelectedManager::ACTION_TYPE_STYLE,
            'tones' => UserFieldsSelectedManager::ACTION_TYPE_TONES,
            'audiences' => UserFieldsSelectedManager::ACTION_TYPE_AUDIENCES,
            'single_editor' => UserFieldsSelectedManager::ACTION_TYPE_SINGLE_EDITOR_FIELDS,
            'generate' => UserFieldsSelectedManager::ACTION_TYPE_BULK_GENERATE_FIELDS,
            'transfer' => UserFieldsSelectedManager::ACTION_TYPE_BULK_TRANSFER_FIELDS,
            'guide_steps' => UserFieldsSelectedManager::ACTION_TYPE_GUIDE_STEPS,
            'gallery' => UserFieldsSelectedManager::ACTION_TYPE_GALLERY
        ];

        try {
            $params = $this->getRequest()->getParams();

            foreach ($params as $param => $value) {
                if (isset($mapping[$param])) {
                    $this->userFieldsSelectedManager->saveUserFieldsSelected(
                        $value,
                        $mapping[$param]
                    );
                }
            }
            
            $result = [
                'success' => true
            ];
        } catch (\Exception $e) {
            $result = [
                'success' => false,
                'message' => $e->getMessage()
            ];
        }

        $response->setData($result);

        return $response;
    }
}
