<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use Magento\Framework\Stdlib\DateTime\TimezoneInterface;

class Keywords extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var TimezoneInterface
     */
    protected $timezone;

    /**
     * Constructor
     *
     * @param TimezoneInterface $timezone
     */
    public function __construct(
        TimezoneInterface $timezone
    ) {
        $this->timezone = $timezone;
    }

    /**
     * Format date
     *
     * @param string $date
     * @param string $locale
     * @return string
     */
    public function formatDate($date, $locale = null)
    {
        $staticDate = new \DateTime($date);
        
        $dateTimeFormatter = $this->timezone->formatDate(
            $staticDate,
            \IntlDateFormatter::MEDIUM,
            $locale
        );

        return $dateTimeFormatter;
    }

    /**
     * Get saved keyword data
     *
     * @param array $savedKeyword
     * @param array $ideas
     * @return array
     */
    public function getSavedKeywordData($savedKeyword, $ideas)
    {
        $found = false;
        
        if (isset($ideas['keywords']) && !empty($ideas['keywords'])) {
            foreach ($ideas['keywords'] as $keyword) {
                if ($keyword['keyword'] === $savedKeyword) {
                    $keyword['search_volume'] = $keyword['search_volume'] ?? __('No data');
                    $keyword['competition'] = $keyword['competition'] ?? __('No data');
                    $savedKeyword = $keyword;
                    $found = true;
                    break;
                }
            }
        }
        
        if (!$found) {
            $savedKeyword = [
                'keyword' => $savedKeyword,
                'search_volume' => __('No data'),
                'competition' => __('No data'),
                'rank_group' => '',
                'intent' => '',
                'serp_infos' => []
            ];
        }

        return $savedKeyword;
    }
}
