<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

namespace WriteTextAI\WriteTextAI\Helper;

use WriteTextAI\WriteTextAI\Helper\Data;

class Store extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var Data
     */
    protected $helper;

    /**
     * Constructor
     *
     * @param Data $helper
     */
    public function __construct(
        Data $helper
    ) {
        $this->helper = $helper;
    }

    /**
     * Get store language
     *
     * @param int $storeId
     * @return string
     */
    public function getStoreLanguage($storeId)
    {
        $locale = $this->helper->conf('general/locale/code', $storeId);

        return $locale;
    }

    /**
     * Get formatted language
     *
     * @param int $storeId
     * @return string
     */
    public function getFormattedLanguage($storeId)
    {
        $language = $this->getStoreLanguage($storeId);
        $language = str_replace('_', '-', $language);

        return $language;
    }

    /**
     * Get region independent language
     *
     * @param int $storeId
     * @return string
     */
    public function getRegionIndependentLanguage($storeId)
    {
        $language = $this->getStoreLanguage($storeId);
        $language = explode('_', (string) $language)[0];

        return $language;
    }

    /**
     * Get formatted region independent language
     *
     * @param int $storeId
     * @return string
     */
    public function getFormattedRegionIndependentLanguage($storeId)
    {
        $language = $this->getFormattedLanguage($storeId);
        $language = explode('-', (string) $language)[0];

        return $language;
    }
}
