<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */
declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordCompetitorFiltersInterface;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordCompetitorFiltersInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordCompetitorFiltersSearchResultsInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordCompetitorFiltersRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordCompetitorFilters
    as ResourceProductKeywordCompetitorFilters;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordCompetitorFilters\CollectionFactory
    as ProductKeywordCompetitorFiltersCollectionFactory;

class ProductKeywordCompetitorFiltersRepository implements ProductKeywordCompetitorFiltersRepositoryInterface
{
    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var ResourceProductKeywordCompetitorFilters
     */
    protected $resource;

    /**
     * @var ProductKeywordCompetitorFiltersInterfaceFactory
     */
    protected $productKeywordCompetitorFiltersFactory;

    /**
     * @var ProductKeywordCompetitorFilters
     */
    protected $searchResultsFactory;

    /**
     * @var ProductKeywordCompetitorFiltersCollectionFactory
     */
    protected $productKeywordCompetitorFiltersCollectionFactory;

    /**
     * @param ResourceProductKeywordCompetitorFilters $resource
     * @param ProductKeywordCompetitorFiltersInterfaceFactory $productKeywordCompetitorFiltersFactory
     * @param ProductKeywordCompetitorFiltersCollectionFactory $productKeywordCompetitorFiltersCollectionFactory
     * @param ProductKeywordCompetitorFiltersSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceProductKeywordCompetitorFilters $resource,
        ProductKeywordCompetitorFiltersInterfaceFactory $productKeywordCompetitorFiltersFactory,
        ProductKeywordCompetitorFiltersCollectionFactory $productKeywordCompetitorFiltersCollectionFactory,
        ProductKeywordCompetitorFiltersSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->productKeywordCompetitorFiltersFactory = $productKeywordCompetitorFiltersFactory;
        $this->productKeywordCompetitorFiltersCollectionFactory = $productKeywordCompetitorFiltersCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritDoc
     */
    public function save(
        ProductKeywordCompetitorFiltersInterface $productKeywordCompetitorFilters
    ) {
        try {
            $this->resource->save($productKeywordCompetitorFilters);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the productKeywordCompetitorFilters: %1',
                $exception->getMessage()
            ));
        }
        return $productKeywordCompetitorFilters;
    }

    /**
     * @inheritDoc
     */
    public function get($productKeywordCompetitorFiltersId)
    {
        $productKeywordCompetitorFilters = $this->productKeywordCompetitorFiltersFactory->create();
        $this->resource->load($productKeywordCompetitorFilters, $productKeywordCompetitorFiltersId);
        if (!$productKeywordCompetitorFilters->getId()) {
            throw new NoSuchEntityException(__(
                'ProductKeywordCompetitorFilters with id "%1" does not exist.',
                $productKeywordCompetitorFiltersId
            ));
        }
        return $productKeywordCompetitorFilters;
    }

    /**
     * @inheritDoc
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->productKeywordCompetitorFiltersCollectionFactory->create();
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model;
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * @inheritDoc
     */
    public function delete(
        ProductKeywordCompetitorFiltersInterface $productKeywordCompetitorFilters
    ) {
        try {
            $productKeywordCompetitorFiltersModel = $this->productKeywordCompetitorFiltersFactory->create();
            $this->resource->load(
                $productKeywordCompetitorFiltersModel,
                $productKeywordCompetitorFilters->getProductkeywordcompetitorfiltersId()
            );
            $this->resource->delete($productKeywordCompetitorFiltersModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the ProductKeywordCompetitorFilters: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * @inheritDoc
     */
    public function deleteById($productKeywordCompetitorFiltersId)
    {
        return $this->delete($this->get($productKeywordCompetitorFiltersId));
    }
}
