<?php
/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

declare(strict_types=1);

namespace WriteTextAI\WriteTextAI\Model;

use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordRankedFiltersInterface;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordRankedFiltersInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\Data\ProductKeywordRankedFiltersSearchResultsInterfaceFactory;
use WriteTextAI\WriteTextAI\Api\ProductKeywordRankedFiltersRepositoryInterface;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordRankedFilters as ResourceProductKeywordRankedFilters;
use WriteTextAI\WriteTextAI\Model\ResourceModel\ProductKeywordRankedFilters\CollectionFactory
    as ProductKeywordRankedFiltersCollectionFactory;

class ProductKeywordRankedFiltersRepository implements ProductKeywordRankedFiltersRepositoryInterface
{
    /**
     * @var ProductKeywordRankedFilters
     */
    protected $searchResultsFactory;

    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var ProductKeywordRankedFiltersCollectionFactory
     */
    protected $productKeywordRankedFiltersCollectionFactory;

    /**
     * @var ProductKeywordRankedFiltersInterfaceFactory
     */
    protected $productKeywordRankedFiltersFactory;

    /**
     * @var ResourceProductKeywordRankedFilters
     */
    protected $resource;

    /**
     * @param ResourceProductKeywordRankedFilters $resource
     * @param ProductKeywordRankedFiltersInterfaceFactory $productKeywordRankedFiltersFactory
     * @param ProductKeywordRankedFiltersCollectionFactory $productKeywordRankedFiltersCollectionFactory
     * @param ProductKeywordRankedFiltersSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceProductKeywordRankedFilters $resource,
        ProductKeywordRankedFiltersInterfaceFactory $productKeywordRankedFiltersFactory,
        ProductKeywordRankedFiltersCollectionFactory $productKeywordRankedFiltersCollectionFactory,
        ProductKeywordRankedFiltersSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->productKeywordRankedFiltersFactory = $productKeywordRankedFiltersFactory;
        $this->productKeywordRankedFiltersCollectionFactory = $productKeywordRankedFiltersCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritDoc
     */
    public function save(
        ProductKeywordRankedFiltersInterface $productKeywordRankedFilters
    ) {
        try {
            $this->resource->save($productKeywordRankedFilters);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the productKeywordRankedFilters: %1',
                $exception->getMessage()
            ));
        }
        return $productKeywordRankedFilters;
    }

    /**
     * @inheritDoc
     */
    public function get($productKeywordRankedFiltersId)
    {
        $productKeywordRankedFilters = $this->productKeywordRankedFiltersFactory->create();
        $this->resource->load($productKeywordRankedFilters, $productKeywordRankedFiltersId);
        if (!$productKeywordRankedFilters->getId()) {
            throw new NoSuchEntityException(__(
                'ProductKeywordRankedFilters with id "%1" does not exist.',
                $productKeywordRankedFiltersId
            ));
        }
        return $productKeywordRankedFilters;
    }

    /**
     * @inheritDoc
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->productKeywordRankedFiltersCollectionFactory->create();
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model;
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * @inheritDoc
     */
    public function delete(
        ProductKeywordRankedFiltersInterface $productKeywordRankedFilters
    ) {
        try {
            $productKeywordRankedFiltersModel = $this->productKeywordRankedFiltersFactory->create();
            $this->resource->load(
                $productKeywordRankedFiltersModel,
                $productKeywordRankedFilters->getProductkeywordrankedfiltersId()
            );
            $this->resource->delete($productKeywordRankedFiltersModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the ProductKeywordRankedFilters: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * @inheritDoc
     */
    public function deleteById($productKeywordRankedFiltersId)
    {
        return $this->delete($this->get($productKeywordRankedFiltersId));
    }
}
