/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/premium',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keyword-filters',
    'WriteTextAI_WriteTextAI/js/model/grid/prompt',
    'WriteTextAI_WriteTextAI/js/model/edit/audience',
    'WriteTextAI_WriteTextAI/js/model/total-credits',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/helpers',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    keywords,
    premium,
    product,
    keywordFilters,
    prompt,
    audience,
    totalCredits,
    keywordProgress,
    helperUtils,
    keywordUtils,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generalMessage: $t('A system error has occurred. Please click the "Start AI-powered analysis" button again. If the issue persists, please contact our support team at support@writetext.ai.'),
            modalSelector: '.keyword-analysis',
            competitionSort: '',
            volumeSort: '',
            keywordSort: '',
            filterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            defaultFilterByCompetition: ['LOW', 'MEDIUM', 'HIGH'],
            filterBySearchVolumeMinimum: 0,
            filterBySearchVolumeMaximum: null,
            filterBySearchVolume: '{"min": 0, "max": null}',
            defaultFilterBySearchVolume: '{"min": 0, "max": null}',
            currentPage: 1,
            pageSize: 10,
            totalPages: 0,
            canFetchData: true,
            isLoading: false
        },

        isFilterByCompetitionChange: false,
        isFilterBySearchVolumeMinimumChange: false,
        isFilterBySearchVolumeMaximumChange: false,
        gettingData: null,

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.keyword-analysis-ideas > *').applyBindings(this);

            keywords.totalPages.subscribe(function (totalPages) {
                self.totalPages(totalPages);
            });

            keywordFilters.ideasSortAndFilters.subscribe(function (data) {
                self.canFetchData = false;
                if (data.hasOwnProperty('competition')) {
                    self.isFilterByCompetitionChange = true;
                    self.filterByCompetition(data.competition);
                    self.isFilterByCompetitionChange = false;
                }
                if (data.hasOwnProperty('search_volume_minimum')) {
                    self.isFilterBySearchVolumeMinimumChange = true;
                    self.filterBySearchVolumeMinimum(
                        data.search_volume_minimum
                    );
                    self.isFilterBySearchVolumeMinimumChange = false;
                }
                if (data.hasOwnProperty('search_volume_maximum')) {
                    self.isFilterBySearchVolumeMaximumChange = true;
                    self.filterBySearchVolumeMaximum(
                        data.search_volume_maximum
                    );
                    self.isFilterBySearchVolumeMaximumChange = false;
                }
                if (data.hasOwnProperty('competition_sort')) {
                    self.competitionSort(data.competition_sort);
                }
                if (data.hasOwnProperty('volume_sort')) {
                    self.volumeSort(data.volume_sort);
                }
                if (data.hasOwnProperty('keyword_sort')) {
                    self.keywordSort(data.keyword_sort);
                }
                self.canFetchData = true;

                self.pageSize(10);
            });

            self.filterBySearchVolume.subscribe(function () {
                var volumesString = self.filterBySearchVolume(),
                    volumesObject = JSON.parse(volumesString);

                self.filterBySearchVolumeMinimum(volumesObject.min);
                self.filterBySearchVolumeMaximum(volumesObject.max);

                self.pageSize(10);
                if (self.canFetchData) {
                    keywords.ideasSortFilterOngoing(true);
                    self.getData();
                }
            });

            self.filterByCompetition.subscribe(function () {
                self.pageSize(10);
                if (self.canFetchData) {
                    keywords.ideasSortFilterOngoing(true);
                    self.getData();
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'competitionSort',
                    'volumeSort',
                    'keywordSort',
                    'filterByCompetition',
                    'filterBySearchVolumeMinimum',
                    'filterBySearchVolumeMaximum',
                    'filterBySearchVolume',
                    'currentPage',
                    'pageSize',
                    'totalPages',
                    'isLoading'
                ]);

            this.suggestedKeywords = ko.computed(function () {
                return keywords.suggestedKeywords();
            });

            this.targetKeywords = ko.computed(function () {
                return keywords.targetKeywords();
            });

            this.hasFilter = ko.computed(function () {
                var hasVolume = this.filterBySearchVolume() !== this.defaultFilterBySearchVolume,
                    hasCompetition = !_.isEqual(
                        this.filterByCompetition().sort(),
                        this.defaultFilterByCompetition.sort()
                    );

                return hasVolume || hasCompetition;
            }, this);

            this.isKeywordsMax = ko.computed(function () {
                var keywords = this.targetKeywords().filter(function (keyword) {
                        return keywordUtils.keywordRemovable(keyword.text);
                }),
                    keywordsCount = keywords ? keywords.length : 0;

                return keywordsCount >= this.rules.maxKeywords;
            }, this);
            
            this.status = ko.computed(function () {
                return keywordProgress.status();
            });

            this.showProgress = ko.computed(function () {
                return Object.keys(this.status()).length > 0 &&
                    keywordProgress.ideasProgressVisible() &&
                    this.status().status !== 'Completed';
            }, this);

            this.getMessage = keywordUtils.getMessage(this);

            this.getCount = keywordUtils.getCount(this);

            this.getPercent = keywordUtils.getPercent(this);

            this.isRefreshing = ko.computed(function () {
                return keywordProgress.ideasLoading();
            });

            this.analysisInProgress = keywordUtils.analysisInProgress();

            this.keywordsChangeOngoing = ko.computed(function () {
                return keywords.keywordsChangeOngoing();
            });

            this.showMoreOngoing = ko.computed(function () {
                return keywords.ideasShowMoreOngoing();
            });

            this.ideasSortFilterOngoing = ko.computed(function () {
                return keywords.ideasSortFilterOngoing();
            });

            this.isSectionDisabled = ko.computed(function () {
                return keywords.ideasSortFilterOngoing() ||
                    keywords.ideasShowMoreOngoing() ||
                    keywords.keywordsChangeOngoing() ||
                    keywords.manualKeywordsChangeOngoing() ||
                    keywordProgress.selectedAndCompetitorLoading();
            });

            this.showRefresh = ko.computed(function () {
                return keywords.showIdeasRefresh();
            }, this);

            this.initialized = ko.computed(function () {
                return keywords.initialized();
            });

            this.noResultText = ko.computed(function () {
                return $t('No keyword data received. Check for misspellings in your keyword(s) or use a different keyword and try again.');
            }); 

            this.stale = ko.computed(function () {
                return keywords.stale();
            });

            this.newKeywordExists = ko.computed(function () {
                return keywords.newKeywordExists();
            });

            this.newTargetExists = ko.computed(function () {
                return keywords.newTargetExists();
            });

            this.targetRemoved = ko.computed(function () {
                return keywords.targetRemoved();
            });

            this.needRefresh = ko.computed(function () {
                return keywords.needRefresh();
            });

            return this;
        },

        /**
         * Check if keyword is removable.
         * 
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordRemovable: function (keyword) {
            return keywordUtils.keywordRemovable(keyword, false);
        },

        /**
         * Dismiss
         */
        dismiss: function () {
            keywordProgress.status({});
            keywordProgress.ideasLoading(false);
            keywordProgress.ideasProgressVisible(false);
        },

        /**
         * Check if keyword is selected.
         *
         * @param {String} keyword
         * @returns {Boolean}
         */
        isKeywordSelected: function (keyword) {
            return keywordUtils.isKeywordSelected(keyword);
        },
    
        /**
         * Refresh data.
         */
        refresh: function () {
            keywords.successMessage('');
            keywordProgress.ideasLoading(true);
            keywordProgress.ideasProgressVisible(true);
            self.resetSortingAndFilters();
            keywordFilters.ideasSortAndFilters({
                competition: ['LOW', 'MEDIUM', 'HIGH'],
                search_volume_minimum: 0,
                search_volume_maximum: null,
                competition_sort: '',
                volume_sort: '',
                keyword_sort: ''
            });
            self.getData(true);
        },

        /**
         * Reset filters
         */
        resetSortingAndFilters: function () {
            self.canFetchData = false;
            self.filterByCompetition([...self.defaultFilterByCompetition]);
            self.filterBySearchVolume(self.defaultFilterBySearchVolume);
            self.filterBySearchVolumeMinimum(0);
            self.filterBySearchVolumeMaximum(null);
            self.volumeSort('');
            self.competitionSort('');
            self.keywordSort('');
            self.canFetchData = true;
        },

        /**
         * Sort by keyword.
         */
        sortByKeyword: function () {
            self.keywordSort(true);
            self.competitionSort('');
            self.volumeSort('');
            keywords.ideasSortFilterOngoing(true);
            self.getData();
        },

        /**
         * Sort by difficulty.
         */
        sortByDifficulty: function () {
            self.keywordSort('');
            self.competitionSort(!self.competitionSort());
            self.volumeSort('');
            keywords.ideasSortFilterOngoing(true);
            self.getData();
        },

        /**
         * Sort by search volume.
         */
        sortBySearchVolume: function () {
            self.keywordSort('');
            self.competitionSort('');
            self.volumeSort(!self.volumeSort());
            keywords.ideasSortFilterOngoing(true);
            self.getData();
        },

        /**
         * Show more keywords.
         */
        showMore: function () {
            self.pageSize(self.pageSize() + 10);
            keywords.ideasShowMoreOngoing(true);
            self.getData();
        },

        /**
         * Close modal.
         */
        closeModal: function () {
            $(self.modalSelector).modal('closeModal');
        },

        /**
         * Get keyword ideas
         *
         * @param {Boolean} refresh
         * @returns {void}
         */
        getData: function (refresh = false) {
            self.isLoading(true);

            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            keywords.errorMessage('');

            if (self.gettingData !== null) {
                self.gettingData.abort();
            }

            self.gettingData = $.ajax({
                url: self.keywordIdeasUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    product_name: product.productName(),
                    store_id: product.storeId(),
                    manual_keywords: keywords.manualInput().map(input => input.keyword),
                    keywords: keywords.targetKeywords()
                        .map(input => input.text)
                        .filter(keyword => keywordUtils.keywordRemovable(keyword)),
                    location: keywords.location(),
                    url: product.productUrl(),
                    competition: self.filterByCompetition(),
                    search_volume_minimum: self.filterBySearchVolumeMinimum(),
                    search_volume_maximum: self.filterBySearchVolumeMaximum(),
                    competition_sort: self.competitionSort(),
                    volume_sort: self.volumeSort(),
                    keyword_sort: self.keywordSort(),
                    page: self.currentPage(),
                    page_size: self.pageSize(),
                    refresh: refresh
                },
                dataType: 'json',
                showWriteTextAILoader: false,
                showAnalysisLoader: false,
                success: function (response) {
                    self.isLoading(false);
                    keywords.ideasShowMoreOngoing(false);
                    keywords.ideasSortFilterOngoing(false);
                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    if (response.success) {
                        if (!refresh) {
                            keywords.manualInput(response.saved_keywords);
                            keywords.suggestedKeywords(response.ideas);
                            self.totalPages(response.total_page);
                            keywordProgress.ideasLoading(false);
                        }
                    } else {
                        keywords.errorMessage(response.message);
                        keywordProgress.selectedAndCompetitorLoading(false);
                        keywordProgress.allSectionsLoading(false);
                        keywordProgress.ideasLoading(false);
                        $('.writetextai-analysis-loading-mask').removeClass('show');
                    }
                },
                error: function (xhr, status, error) {
                    self.isLoading(false);
                    if (status !== 'abort') {
                        keywords.errorMessage(error);
                    }
                }
            });
        },

        /**
         * Add keyword to product.
         * 
         * @param {Object} data
         * @param {Event} event
         * 
         * @returns {void}
         */
        addKeyword: function (data, event) {
            var targetKeywords = self
                    .targetKeywords()
                    .map((item) => item['text']),
                element = event.currentTarget,
                parentElement = $(element).closest('tr'),
                keywordData = {
                    text: this.keyword,
                    values: [],
                    temp: true,
                    search_volume: this.search_volume,
                    competition: this.competition,
                    rank_group: this.rank_group,
                    intent: this.intent
            };

            if (self.isKeywordsMax()) {
                return;
            }
                
            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            keywords.errorMessage('');
            /* Validate keyword length */
            if (keywordData.text.length > self.rules.maxKeywordLength) {
                keywords.errorMessage('Target keyword is too long.');
                return;
            }
            /* Validate duplicate keyword */
            var isDuplicate = self.targetKeywords().find(keyword => {
                return keyword.text.toLowerCase() === keywordData.text.toLowerCase();
            });
            if (isDuplicate) {
                keywords.errorMessage($t('The entered keyword already exists.'));
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.targetKeywords.push(keywordData);

            $.ajax({
                url: self.keywordAddUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    keywords: targetKeywords,
                    keyword: keywordData.text
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                hideWriteTextTitleLoader: true,
                showKeywordsChangeLoader: true,
                success: function (response) {
                    var newKeyword = {};

                    $(parentElement).find('.action').removeClass('loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }

                    if (response.success) {
                        const oldKeyword = ko.utils.arrayFirst(
                            keywords.targetKeywords(),
                            function (data) {
                                return (
                                    data.text === response.data.text &&
                                    data.temp === true
                                );
                            }
                        );
                        
                        if (oldKeyword) {
                            newKeyword = {...oldKeyword};
                            newKeyword.values = response.data.values;
                            keywords.targetKeywords.replace(oldKeyword, newKeyword);
                        } else {
                            keywords.targetKeywords.push(response.data);
                        }

                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.newTargetExists(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.remove(function (data) {
                            return (
                                data.text === keywordData.text &&
                                data.temp === true
                            );
                        });
                    }
                }
            });

            return true;
        },
        
        /**
         * Remove keyword from product.
         * 
         * @param {Object} data
         * @param {Event} event
         * 
         * @returns {void}
         */
        removeKeyword: function (data, event) {
            var element = event.currentTarget,
                parentElement = $(element).closest('tr');

            if (!premium.isPremium()) {
                self.closeModal();
                return;
            }

            $(parentElement).find('.action').addClass('loading');

            keywords.errorMessage('');
            
            let selectedKeyword = this.keyword;
            let keywordToRemove = ko.utils.arrayFirst(
                keywords.targetKeywords(),
                function (data) {
                    return (
                        data.text === selectedKeyword
                    );
                }
            );

            if (keywordToRemove) {
                keywords.targetKeywords.remove(keywordToRemove);
            }

            $.ajax({
                url: self.keywordDeleteUrl,
                type: 'POST',
                data: {
                    product_id: product.productId(),
                    store_id: product.storeId(),
                    remaining_keywords: keywords.targetKeywords()
                        .map(function (keyword) {
                            return { text: keyword.text };
                        })
                },
                dataType: 'json',
                showWriteTextAILoader: true,
                hideWriteTextTitleLoader: true,
                showKeywordsChangeLoader: true,
                success: function (response) {
                    $(parentElement).find('.action').removeClass('loading');

                    if (typeof response.is_premium !== 'undefined') {
                        premium.isPremium(response.is_premium);
                        prompt.isPremium(response.is_premium);
                        totalCredits.totalCredits(response.total_credits);
                    }
                    
                    if (response.success) {
                        keywords.selectedSemanticKeywords(response.selected);
                        audience.selectedAudience(response.selected_audience.shift());
                        audience.suggestedAudience(response.suggested_audience);
                        keywords.targetRemoved(true);
                    } else {
                        keywords.errorMessage(response.message);
                        keywords.targetKeywords.push(keywordToRemove);
                    }
                }.bind(this)
            });
        },
    });
});
