/**
 * @author 1902 Software
 * @copyright Copyright © 2023 1902 Software (https://1902software.com/magento/)
 * @package WriteTextAI_WriteTextAI
 */

define([
    'jquery',
    'ko',
    'uiComponent',
    'underscore',
    'WriteTextAI_WriteTextAI/js/model/edit/product',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords',
    'WriteTextAI_WriteTextAI/js/model/edit/keywords/progress',
    'WriteTextAI_WriteTextAI/js/utils/edit/keywords',
    'WriteTextAI_WriteTextAI/js/edit/keyword-analysis',
    'mage/translate'
], function (
    $,
    ko,
    Component,
    _,
    product,
    keywords,
    keywordProgress,
    keywordUtils,
    keywordAnalysisComponent,
    $t
) {
    'use strict';

    /**
     * @var {Component} self
     */
    var self;

    return Component.extend({
        defaults: {
            generalMessage: $t('A system error has occurred. Please click the "Start AI-powered analysis" button again. If the issue persists, please contact our support team at support@writetext.ai.'),
            isLoading: false
        },

        /** @inheritdoc */
        initialize: function () {
            this._super();

            self = this;

            $('.keyword-analysis-progress > *').applyBindings(this);

            self.status.subscribe(function (status) {
                if (status.status === 'Completed') {
                    keywordAnalysisComponent.prototype.getData();
                }
            });
        },

        /** @inheritdoc */
        initObservable: function () {
            this._super()
                .observe([
                    'isLoading'
                ]);
            
            this.status = ko.computed(function () {
                return keywordProgress.status();
            });

            this.showProgress = ko.computed(function () {
                this.showProgressInOtherUsers();
                
                return Object.keys(this.status()).length > 0 &&
                    keywordProgress.allSectionsProgressVisible() &&
                    this.status().status !== 'Completed';
            }, this);

            this.statusStatus = ko.computed(function () {
                if (Object.keys(this.status()).length === 0) {
                    return '';
                } else {
                    return this.status().status;
                }
            }, this);

            this.statusStatus.subscribe(function (status) {
                if (status !== 'Running') {
                    keywordProgress.selectedAndCompetitorLoading(false);
                    keywordProgress.allSectionsLoading(false);
                    keywordProgress.ideasLoading(false);
                    keywordProgress.progressFromOtherUserOngoing(false);
                    $('.writetextai-analysis-loading-mask').removeClass('show');
                    this.isLoading(false);
                }
                if (status === 'Completed') {
                    keywordProgress.selectedAndCompetitorProgressVisible(false);
                    keywordProgress.allSectionsProgressVisible(false);
                    keywordProgress.ideasProgressVisible(false);

                    if (Object.keys(this.status()).length > 0 &&
                        this.status().status_display !== ''
                    ) {
                        keywords.successMessage(this.status().status_display);
                    }
                }
            }, this);

            this.getMessage = keywordUtils.getMessage(this);

            this.getCount = keywordUtils.getCount(this);

            this.getPercent = keywordUtils.getPercent(this);

            return this;
        },

        /**
         * Show progress in other users
         */
        showProgressInOtherUsers: function () {
            let notLoading = !this.isLoading(),
                running = this.status().status === 'Running',
                hasStatus = Object.keys(this.status()).length > 0,
                isCurrentProduct = parseInt(this.status().recordId) === parseInt(product.productId()),
                sectionsNotLoading =
                    !keywordProgress.allSectionsLoading() &&
                    !keywordProgress.selectedAndCompetitorLoading() &&
                    !keywordProgress.ideasLoading();

            if (hasStatus && sectionsNotLoading && running && notLoading && isCurrentProduct) {
                $('.writetextai-analysis-loading-mask').addClass('show');
                keywordProgress.allSectionsLoading(true);
                keywordProgress.allSectionsProgressVisible(true);
                keywordProgress.progressFromOtherUserOngoing(true);
                this.isLoading(true);
            }
        },

        /**
         * Dismiss
         */
        dismiss: function () {
            keywordProgress.status({});
            keywordProgress.allSectionsLoading(false);
            keywordProgress.allSectionsProgressVisible(false);
            $('.writetextai-analysis-loading-mask').removeClass('show');
        }
    });
});
